@@ -1,4 +1,4 @@
-.CV.nnet = function(Input, Target, size=c(2,4,6, 8), decay=c(0.001, 0.01, 0.05, 0.1), nbCV=5, W=NULL){
+.CV.nnet = function(Input, Target, size=c(2,4,6, 8), decay=c(0.001, 0.01, 0.05, 0.1), maxit=200, nbCV=5, W=NULL){
 #   require(pROC, quietly=T)
 
   Eval = data.frame(matrix(0, ncol=3, nrow=16, dimnames=list(NULL, c("Size", "Decay", "AUC"))))
@@ -12,15 +12,15 @@
           Eval[,3] = Eval[,3] + apply(Eval[,1:2], 1, Samp, Target, Input, FUN=function(x, Samp, Target, Input){
             nn = nnet(eval(parse(text = paste("Target[Samp$calibration]",
                   paste(.scopeExpSyst(Input[1:10, ,drop=FALSE], "GBM"), collapse = "")))),data=Input[Samp$calibration, ,drop=FALSE],
-                  size = x[1], decay = x[2], maxit = 200, trace = FALSE)
+                  size = x[1], decay = x[2], maxit = maxit, trace = FALSE)
             AUC = as.numeric(pROC::auc(pROC::roc(Target[Samp$evaluation], predict(nn, Input[Samp$evaluation,,drop=FALSE]))))
             return(AUC)
           })
       } else{
           Eval[,3] = Eval[,3] + apply(Eval[,1:2], 1, Samp, Target, Input, W, FUN=function(x, Samp, Target, Input, W){
-            nn = nnet(eval(parse(text = paste("Target[Samp$calibration]",
+          nn = nnet(eval(parse(text = paste("Target[Samp$calibration]",
                   paste(.scopeExpSyst(Input[1:10, ,drop=FALSE], "GBM"), collapse = "")))),data=Input[Samp$calibration, ,drop=FALSE],
-                  weights=W[Samp$calibration], size = x[1], decay = x[2], maxit = 200, trace = FALSE)
+                  weights=W[Samp$calibration], size = x[1], decay = x[2], maxit = maxit, trace = FALSE)
             AUC = as.numeric(pROC::auc(pROC::roc(Target[Samp$evaluation], predict(nn, Input[Samp$evaluation,,drop=FALSE]))))
             return(AUC)
           })
